﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Data;

using Infragistics.Windows.DataPresenter;

using ViewModels;

namespace CustomControlForXamDataGrid
{
    /// <summary>
    /// Interaction logic for ChannelDataGrid.xaml
    /// </summary>
    public partial class ChannelDataGrid : XamDataGrid
    {
       
        private List<ParameterViewModel> _selectedCells=new List<ParameterViewModel>();
      

        #region Dependency Properties

        /// <summary>
        /// Identifies the <c>IsManaged</c> attached dependency property.  This field is read-only.
        /// </summary>
        /// <value>The identifier for the <c>IsManaged</c> attached dependency property.</value>
        public static readonly DependencyProperty ParametersOrderListProperty = DependencyProperty.RegisterAttached(
            "ParametersOrderList",
            typeof(List<string>),
            typeof(ChannelDataGrid),
            new PropertyMetadata(null, OnParametersOrderListPropertyPropertyValueChanged));

        public static readonly DependencyProperty SelectedCellsProperty = DependencyProperty.RegisterAttached(
           "SelectedCells",
           typeof(List<ParameterViewModel>),
           typeof(ChannelDataGrid));        

        #endregion // Dependency Properties

        /// <summary>
        /// Initializes a new instance of the <see cref="ChannelDataGrid"/> class.
        /// </summary>
        public ChannelDataGrid()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Gets or sets the parameters list.
        /// </summary>
        /// <value>The parameters list.</value>
        public List<string> ParametersOrderList
        {
            get
            {
                return (List<string>)GetValue(ParametersOrderListProperty);
            }

            set
            {
                SetValue(ParametersOrderListProperty, value);
            }
        }

        public List<ParameterViewModel> SelectedCells
        {
            get
            {
                //return (List<ParameterViewModel>)GetValue(SelectedCellsProperty);
                return this._selectedCells;
            }

            set
            {
                SetValue(SelectedCellsProperty, value);
            }
        }

        /// <summary>
        /// Called when [parameters list property value changed].
        /// </summary>
        /// <param name="d">The d.</param>
        /// <param name="e">The <see cref="System.Windows.DependencyPropertyChangedEventArgs"/> instance containing the event data.</param>
        private static void OnParametersOrderListPropertyPropertyValueChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var channelDataGrid = (ChannelDataGrid)d;

            for (int index = 0; index < channelDataGrid.ParametersOrderList.Count; index++)
            {
                var parameter = channelDataGrid.ParametersOrderList[index];
                UnboundField parameterField = new UnboundField();
                parameterField.BindingPath = new PropertyPath("Parameters[" + parameter + "].Value", null);
                parameterField.BindingMode = BindingMode.TwoWay;
                parameterField.Name = parameter;
                parameterField.Label = parameter;
                
                parameterField.DataType = typeof(string);
                //parameterField.Settings.CellValuePresenterStyle = CreateCellStryle(parameter);
                var binding = new Binding() {Path = new PropertyPath("Parameters[" + parameter + "].Width"), Mode = BindingMode.TwoWay };
                
                channelDataGrid.FieldLayouts[0].Fields.Add(parameterField);
            }
        }

        /// <summary>
        /// Creates the na cell stryle.
        /// </summary>
        /// <param name="parameterName">Name of the parameter.</param>
        /// <returns>The style.</returns>
        private static Style CreateCellStryle(string parameterName)
        {
            var cellStryle = new Style { TargetType = typeof(CellValuePresenter) };
            var enableSetter = new Setter(
                IsEnabledProperty,
                new Binding("DataItem.Parameters[" + parameterName + "].IsEnabled")
                {
                    Mode = BindingMode.OneWay
                });

            cellStryle.Setters.Add(enableSetter);

            return cellStryle;
        }           

        private void XamDataGrid_EditModeStarting(object sender, Infragistics.Windows.DataPresenter.Events.EditModeStartingEventArgs e)
        {
            ChannelViewModel vm = e.Cell.Record.DataItem as ChannelViewModel;

            if (!vm.Parameters[e.Cell.Field.Name].IsEnabled)
                e.Cancel = true;
        }
    }
}
