﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading;

using ViewModels;
using Infragistics.Windows.DataPresenter;

namespace CustomControlForXamDataGrid.ViewModel
{
    /// <summary>
    /// TODO: Update summary.
    /// </summary>
    public class MainWindowViewModel : ViewModelBase.ViewModelBase
    {
        /// <summary>
        /// The channels.
        /// </summary>
        private List<ChannelViewModel> _channels;

        /// <summary>
        /// The order for the parameters.
        /// </summary>
        private List<string> _parametersOrderList;

        /// <summary>
        /// Initializes a new instance of the <see cref="MainWindowViewModel"/> class.
        /// </summary>
        public MainWindowViewModel()
        {
            Initialize();

            HandleParametersOfChannel();
        }

        /// <summary>
        /// Gets or sets the channels.
        /// </summary>
        /// <value>The channels.</value>
        public List<ChannelViewModel> Channels
        {
            get
            {
                return _channels;
            }

            set
            {
                _channels = value;
                OnPropertyChanged("Channels");
            }
        }

        /// <summary>
        /// Gets or sets the parameters order list.
        /// </summary>
        /// <value>The parameters order list.</value>
        public List<string> ParametersOrderList
        {
            get
            {
                return _parametersOrderList;
            }

            set
            {
                _parametersOrderList = value;
                OnPropertyChanged("ParametersOrderList");
            }
        }

        public List<ParameterViewModel> _selectedCells;
        public List<ParameterViewModel> SelectedCells
        {
            get
            {
                return this._selectedCells;
            }

            set
            {
                this._selectedCells = value;
                OnPropertyChanged("SelectedCells");
            }
        }

        private bool _wrapText = true;
        public bool WrapText
        {
            get
            {
                return this._wrapText;
            }

            set
            {
                this._wrapText = value;
                OnPropertyChanged("WrapText");
            }
        }

        /// <summary>
        /// Initializes this instance.
        /// </summary>
        private void Initialize()
        {
            Channels = new List<ChannelViewModel>();

            ParametersOrderList = new List<string>();

            for (int i = 0; i < 5; i++)
            {
                var parameterViewModels = new Dictionary<string, ParameterViewModel>();

                for (int index = 0; index < 10; index++)
                {
                    var paraIndex = index + 1;
                    if (!ParametersOrderList.Contains("Parameter " + paraIndex))
                    {
                        ParametersOrderList.Add("Parameter " + paraIndex);
                    }

                    var isEnabled = (paraIndex % 2 == 0);
                    parameterViewModels.Add(
                                "Parameter " + paraIndex,
                                new ParameterViewModel
                                    {
                                        ParameterName = "Parameter " + paraIndex,
                                        Value = "ABCDFREERWEWE",
                                        IsEnabled = isEnabled
                                    });
                }

                Channels.Add(new ChannelViewModel { Name = (i + 1).ToString(), Parameters = parameterViewModels });
            }
        }

        /// <summary>
        /// Handles the parameters of channel.
        /// </summary>
        private void HandleParametersOfChannel()
        {
            ParametersOrderList = ParametersOrderList.OrderBy(parameter => parameter).ToList();

            foreach (var channelViewModel in Channels)
            {
                foreach (var parameter in ParametersOrderList)
                {
                    if (!channelViewModel.Parameters.ContainsKey(parameter))
                    {
                        channelViewModel.Parameters.Add(
                            parameter, new ParameterViewModel { ParameterName = parameter, Value = "DDDDDDDFEWQ" });
                    }
                }
            }
        }
    }
}
